@@ -1,29 +1,19 @@
-# std lib
-from typing import Optional
+import dask.array as da
 from functools import partial
 import math
-
-# 3rd-party
-try:
-    import cupy
-except ImportError:
-    class cupy(object):
-        ndarray = False
-
-import dask.array as da
-
 from numba import cuda
-
 import numpy as np
+from typing import Optional
 import xarray as xr
 
-# local modules
-from xrspatial.utils import cuda_args
-from xrspatial.utils import has_cuda
-from xrspatial.utils import is_cupy_backed
+from .gpu_rtx import has_rtx
+from .utils import (
+    calc_cuda_dims, has_cuda, has_cupy, is_cupy_array, is_cupy_backed)
 
 
 def _run_numpy(data, azimuth=225, angle_altitude=25):
+    data = data.astype(np.float32)
+
     azimuth = 360.0 - azimuth
     x, y = np.gradient(data)
     slope = np.pi/2. - np.arctan(np.sqrt(x*x + y*y))
@@ -40,6 +30,8 @@ def _run_numpy(data, azimuth=225, angle_altitude=25):
 
 
 def _run_dask_numpy(data, azimuth, angle_altitude):
+    data = data.astype(np.float32)
+
     _func = partial(_run_numpy, azimuth=azimuth, angle_altitude=angle_altitude)
     out = data.map_overlap(_func,
                            depth=(1, 1),
@@ -48,11 +40,6 @@ def _run_dask_numpy(data, azimuth, angle_altitude):
     return out
 
 
-def _run_dask_cupy(data, azimuth, angle_altitude):
-    msg = 'Not implemented.'
-    raise NotImplementedError(msg)
-
-
 @cuda.jit
 def _gpu_calc_numba(
     data,
@@ -90,8 +77,10 @@ def _run_cupy(d_data, azimuth, angle_altitude):
     azimuthrad = (360.0 - azimuth) * np.pi / 180.
 
     # Allocate output buffer and launch kernel with appropriate dimensions
+    import cupy
+    d_data = d_data.astype(cupy.float32)
     output = cupy.empty(d_data.shape, np.float32)
-    griddim, blockdim = cuda_args(d_data.shape)
+    griddim, blockdim = calc_cuda_dims(d_data.shape)
     _gpu_calc_numba[griddim, blockdim](
         d_data, output, sin_altituderad, cos_altituderad, azimuthrad
     )
@@ -108,7 +97,8 @@ def _run_cupy(d_data, azimuth, angle_altitude):
 def hillshade(agg: xr.DataArray,
               azimuth: int = 225,
               angle_altitude: int = 25,
-              name: Optional[str] = 'hillshade') -> xr.DataArray:
+              name: Optional[str] = 'hillshade',
+              shadows: bool = False) -> xr.DataArray:
     """
     Calculates, for all cells in the array, an illumination value of
     each cell based on illumination from a specific azimuth and
@@ -127,6 +117,10 @@ def hillshade(agg: xr.DataArray,
         specified in degrees.
     name : str, default='hillshade'
         Name of output DataArray.
+    shadows : bool, default=False
+        Whether to calculate shadows or not. Shadows are available
+        only for Cupy-backed Dask arrays and only if rtxpy is
+        installed and appropriate graphics hardware is available.
 
     Returns
     -------
@@ -139,29 +133,21 @@ def hillshade(agg: xr.DataArray,
 
     Examples
     --------
-    .. plot::
-       :include-source:
-
-            import numpy as np
-            import xarray as xr
-            from xrspatial import hillshade
-
-            data = np.array([
-                [0., 0., 0., 0., 0.],
-                [0., 1., 0., 2., 0.],
-                [0., 0., 3., 0., 0.],
-                [0., 0., 0., 0., 0.],
-                [0., 0., 0., 0., 0.]
-            ])
-            n, m = data.shape
-            raster = xr.DataArray(data, dims=['y', 'x'], name='raster')
-            raster['y'] = np.arange(n)[::-1]
-            raster['x'] = np.arange(m)
-
-            hillshade_agg = hillshade(raster)
-
     .. sourcecode:: python
-
+        >>> import numpy as np
+        >>> import xarray as xr
+        >>> from xrspatial import hillshade
+        >>> data = np.array([
+        ...    [0., 0., 0., 0., 0.],
+        ...    [0., 1., 0., 2., 0.],
+        ...    [0., 0., 3., 0., 0.],
+        ...    [0., 0., 0., 0., 0.],
+        ...    [0., 0., 0., 0., 0.]])
+        >>> n, m = data.shape
+        >>> raster = xr.DataArray(data, dims=['y', 'x'], name='raster')
+        >>> raster['y'] = np.arange(n)[::-1]
+        >>> raster['x'] = np.arange(m)
+        >>> hillshade_agg = hillshade(raster)
         >>> print(hillshade_agg)
         <xarray.DataArray 'hillshade' (y: 5, x: 5)>
         array([[       nan,        nan,        nan,        nan,        nan],
@@ -174,17 +160,26 @@ def hillshade(agg: xr.DataArray,
           * x        (x) int32 0 1 2 3 4
     """
 
+    if shadows and not has_rtx():
+        raise RuntimeError(
+            "Can only calculate shadows if cupy and rtxpy are available")
+
     # numpy case
     if isinstance(agg.data, np.ndarray):
         out = _run_numpy(agg.data, azimuth, angle_altitude)
 
     # cupy/numba case
-    elif has_cuda() and isinstance(agg.data, cupy.ndarray):
-        out = _run_cupy(agg.data, azimuth, angle_altitude)
+    elif has_cuda() and has_cupy() and is_cupy_array(agg.data):
+        if shadows and has_rtx():
+            from .gpu_rtx.hillshade import hillshade_rtx
+            out = hillshade_rtx(agg, azimuth, angle_altitude, shadows=shadows)
+        else:
+            out = _run_cupy(agg.data, azimuth, angle_altitude)
 
     # dask + cupy case
-    elif has_cuda() and isinstance(agg.data, da.Array) and is_cupy_backed(agg):
-        out = _run_dask_cupy(agg.data, azimuth, angle_altitude)
+    elif (has_cuda() and has_cupy() and isinstance(agg.data, da.Array) and
+            is_cupy_backed(agg)):
+        raise NotImplementedError("Dask/CuPy hillshade not implemented")
 
     # dask + numpy case
     elif isinstance(agg.data, da.Array):
