@@ -16,21 +16,22 @@ from numba import cuda, jit
 import numba as nb
 
 # local modules
-from xrspatial.utils import has_cuda
 from xrspatial.utils import cuda_args
+from xrspatial.utils import ArrayTypeFunctionMapping
+from xrspatial.utils import not_implemented_func
 
 
-@jit(nopython=True, nogil=True, parallel=True, cache=True)
+@jit(nopython=True, nogil=True)
 def _lerp(a, b, x):
     return a + x * (b - a)
 
 
-@jit(nopython=True, nogil=True, parallel=True, cache=True)
+@jit(nopython=True, nogil=True)
 def _fade(t):
     return 6 * t ** 5 - 15 * t ** 4 + 10 * t ** 3
 
 
-@jit(nopython=True, nogil=True, parallel=True, cache=True)
+@jit(nopython=True, nogil=True)
 def _gradient(h, x, y):
     # assert(len(h.shape) == 2)
     vectors = np.array([[0, 1], [0, -1], [1, 0], [-1, 0]])
@@ -210,22 +211,18 @@ def perlin(agg: xr.DataArray,
 
     Examples
     --------
-    .. plot::
-       :include-source:
-
-        import numpy as np
-        import xarray as xr
-        from xrspatial import perlin
-
-        W = 4
-        H = 3
-        data = np.zeros((H, W), dtype=np.float32)
-        raster = xr.DataArray(data, dims=['y', 'x'])
-
-        perlin_noise = perlin(raster)
-
     .. sourcecode:: python
 
+        >>> import numpy as np
+        >>> import xarray as xr
+        >>> from xrspatial import perlin
+
+        >>> W = 4
+        >>> H = 3
+        >>> data = np.zeros((H, W), dtype=np.float32)
+        >>> raster = xr.DataArray(data, dims=['y', 'x'])
+
+        >>> perlin_noise = perlin(raster)
         >>> print(perlin_noise)
         <xarray.DataArray 'perlin' (y: 3, x: 4)>
         array([[0.39268944, 0.27577767, 0.01621884, 0.05518942],
@@ -234,18 +231,15 @@ def perlin(agg: xr.DataArray,
         Dimensions without coordinates: y, x
     """
 
-    # numpy case
-    if isinstance(agg.data, np.ndarray):
-        out = _perlin_numpy(agg.data, freq, seed)
-    # cupy case
-    elif has_cuda() and isinstance(agg.data, cupy.ndarray):
-        out = _perlin_cupy(agg.data, freq, seed)
-    # dask + numpy case
-    elif isinstance(agg.data, da.Array):
-        out = _perlin_dask_numpy(agg.data, freq, seed)
-    else:
-        raise TypeError('Unsupported Array Type: {}'.format(type(agg.data)))
-
+    mapper = ArrayTypeFunctionMapping(
+        numpy_func=_perlin_numpy,
+        cupy_func=_perlin_cupy,
+        dask_func=_perlin_dask_numpy,
+        dask_cupy_func=lambda *args: not_implemented_func(
+            *args, messages='perlin() does not support dask with cupy backed DataArray',  # noqa
+        )
+    )
+    out = mapper(agg)(agg.data, freq, seed)
     result = xr.DataArray(out,
                           dims=agg.dims,
                           attrs=agg.attrs,
