@@ -5,7 +5,8 @@ import xarray as xr
 
 from xrspatial import aspect
 from xrspatial.utils import doesnt_have_cuda
-from xrspatial.utils import is_cupy_backed
+
+from xrspatial.tests.general_checks import general_output_checks
 
 
 INPUT_DATA = np.asarray([
@@ -35,26 +36,28 @@ def test_numpy_equals_qgis():
     small_da = xr.DataArray(INPUT_DATA, attrs={'res': (10.0, 10.0)})
     xrspatial_aspect = aspect(small_da, name='numpy_aspect')
 
-    # validate output attributes
-    assert xrspatial_aspect.dims == small_da.dims
-    assert xrspatial_aspect.attrs == small_da.attrs
-    assert xrspatial_aspect.shape == small_da.shape
+    general_output_checks(small_da, xrspatial_aspect)
     assert xrspatial_aspect.name == 'numpy_aspect'
-    for coord in small_da.coords:
-        assert np.all(xrspatial_aspect[coord] == small_da[coord])
 
-    # TODO: We shouldn't ignore edges!
     # validate output values
-    # ignore border edges
-    xrspatial_vals = xrspatial_aspect.values[1:-1, 1:-1]
+    xrspatial_vals = xrspatial_aspect.data[1:-1, 1:-1]
     qgis_vals = QGIS_OUTPUT[1:-1, 1:-1]
-
-    # TODO: use np.is_close instead
-    # set a tolerance of 1e-4
     # aspect is nan if nan input
-    # aspect is invalid (nan) if slope equals 0
+    # aspect is invalid (-1) if slope equals 0
     # otherwise aspect are from 0 - 360
-    assert np.isclose(xrspatial_vals, qgis_vals, equal_nan=True).all()
+    np.testing.assert_allclose(xrspatial_vals, qgis_vals, equal_nan=True)
+
+    # nan edge effect
+    xrspatial_edges = [
+        xrspatial_aspect.data[0, :],
+        xrspatial_aspect.data[-1, :],
+        xrspatial_aspect.data[:, 0],
+        xrspatial_aspect.data[:, -1],
+    ]
+    for edge in xrspatial_edges:
+        np.testing.assert_allclose(
+            edge, np.full(edge.shape, np.nan), equal_nan=True
+        )
 
 
 def test_numpy_equals_dask():
@@ -68,15 +71,13 @@ def test_numpy_equals_dask():
     numpy_result = aspect(small_numpy_based_data_array, name='numpy_result')
     dask_result = aspect(small_dask_based_data_array,
                          name='dask_result')
-
-    assert isinstance(dask_result.data, da.Array)
-
-    dask_result.data = dask_result.data.compute()
-    assert np.isclose(numpy_result, dask_result, equal_nan=True).all()
+    general_output_checks(small_dask_based_data_array, dask_result)
+    np.testing.assert_allclose(
+        numpy_result.data, dask_result.data.compute(), equal_nan=True)
 
 
 @pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
-def test_numpy_equals_cupy():
+def test_cpu_equals_gpu():
 
     import cupy
 
@@ -87,54 +88,5 @@ def test_numpy_equals_cupy():
     # aspect by xrspatial
     cpu = aspect(small_da, name='aspect_agg')
     gpu = aspect(small_da_cupy, name='aspect_agg')
-
-    assert isinstance(gpu.data, cupy.ndarray)
-    assert np.isclose(cpu, gpu.data.get(), equal_nan=True).all()
-
-
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
-def test_cupy_equals_qgis():
-
-    import cupy
-
-    small_da = xr.DataArray(INPUT_DATA, attrs={'res': (10.0, 10.0)})
-    small_da_cupy = xr.DataArray(cupy.asarray(INPUT_DATA),
-                                 attrs={'res': (10.0, 10.0)})
-    xrspatial_aspect = aspect(small_da_cupy, name='aspect_agg')
-
-    # validate output attributes
-    assert xrspatial_aspect.dims == small_da.dims
-    assert xrspatial_aspect.attrs == small_da.attrs
-    assert xrspatial_aspect.shape == small_da.shape
-    assert xrspatial_aspect.name == 'aspect_agg'
-    for coord in small_da.coords:
-        assert np.all(xrspatial_aspect[coord] == small_da[coord])
-
-    # TODO: We shouldn't ignore edges!
-    # validate output values
-    # ignore border edges
-    xrspatial_vals = xrspatial_aspect.data[1:-1, 1:-1].get()
-    qgis_vals = QGIS_OUTPUT[1:-1, 1:-1]
-    assert np.isclose(xrspatial_vals, qgis_vals, equal_nan=True).all()
-
-
-@pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
-def _numpy_equals_dask_cupy():
-
-    # NOTE: Dask + GPU code paths don't currently work because of
-    # dask casting cupy arrays to numpy arrays during
-    # https://github.com/dask/dask/issues/4842
-
-    import cupy
-
-    cupy_data = cupy.asarray(INPUT_DATA)
-    dask_cupy_data = da.from_array(cupy_data, chunks=(3, 3))
-
-    small_da = xr.DataArray(INPUT_DATA, attrs={'res': (10.0, 10.0)})
-    cpu = aspect(small_da, name='numpy_result')
-
-    small_dask_cupy = xr.DataArray(dask_cupy_data, attrs={'res': (10.0, 10.0)})
-    gpu = aspect(small_dask_cupy, name='cupy_result')
-
-    assert is_cupy_backed(gpu)
-    assert np.isclose(cpu, gpu, equal_nan=True).all()
+    general_output_checks(small_da_cupy, gpu)
+    np.testing.assert_allclose(cpu.data, gpu.data.get(), equal_nan=True)
