@@ -8,6 +8,8 @@ from xrspatial.utils import has_cuda
 from xrspatial.utils import doesnt_have_cuda
 from xrspatial import perlin
 
+from xrspatial.tests.general_checks import general_output_checks
+
 
 def create_test_arr(backend='numpy'):
     W = 50
@@ -29,17 +31,17 @@ def test_perlin_cpu():
     # vanilla numpy version
     data_numpy = create_test_arr()
     perlin_numpy = perlin(data_numpy)
+    general_output_checks(data_numpy, perlin_numpy)
 
     # dask
     data_dask = create_test_arr(backend='dask')
     perlin_dask = perlin(data_dask)
-    assert isinstance(perlin_dask.data, da.Array)
+    general_output_checks(data_dask, perlin_dask)
 
-    perlin_dask = perlin_dask.compute()
-    assert np.isclose(
-        perlin_numpy.data, perlin_dask.data,
+    np.testing.assert_allclose(
+        perlin_numpy.data, perlin_dask.data.compute(),
         rtol=1e-05, atol=1e-07, equal_nan=True
-    ).all()
+    )
 
 
 @pytest.mark.skipif(doesnt_have_cuda(), reason="CUDA Device not Available")
@@ -51,8 +53,8 @@ def test_perlin_gpu():
     # cupy
     data_cupy = create_test_arr(backend='cupy')
     perlin_cupy = perlin(data_cupy)
-
-    assert np.isclose(
-        perlin_numpy.data, perlin_cupy.data,
+    general_output_checks(data_cupy, perlin_cupy)
+    np.testing.assert_allclose(
+        perlin_numpy.data, perlin_cupy.data.get(),
         rtol=1e-05, atol=1e-07, equal_nan=True
-    ).all()
+    )
