@@ -150,12 +150,13 @@ def test_crosstab_2d():
     # numpy case
     zones_np, values_np, _ = create_zones_values(backend='numpy')
     df_np = crosstab(
-        zones=zones_np, values=values_np, nodata_zones=0, nodata_values=3
+        zones=zones_np, values=values_np,
+        zone_ids=[1, 2, 3], cat_ids=[0, 1, 2],
     )
     # dask case
     zones_da, values_da, _ = create_zones_values(backend='dask')
     df_da = crosstab(
-        zones=zones_da, values=values_da, nodata_zones=0, nodata_values=3
+        zones=zones_da, values=values_da, zone_ids=[1, 2, 3], nodata_values=3
     )
     check_results(df_np, df_da, crosstab_2d_results)
 
@@ -170,12 +171,12 @@ def test_crosstab_2d():
     # numpy case
     df_np = crosstab(
         zones=zones_np, values=values_np, zone_ids=[1, 2], cat_ids=[1, 2],
-        nodata_zones=0, nodata_values=3, agg='percentage'
+        nodata_values=3, agg='percentage'
     )
     # dask case
     df_da = crosstab(
         zones=zones_da, values=values_da, zone_ids=[1, 2], cat_ids=[1, 2],
-        nodata_zones=0, nodata_values=3, agg='percentage'
+        nodata_values=3, agg='percentage'
     )
     check_results(df_np, df_da, crosstab_2d_percentage_results)
 
@@ -216,13 +217,13 @@ def test_crosstab_3d():
     zones_np, _, values_np = create_zones_values(backend='numpy')
     df_np = crosstab(
         zones=zones_np, values=values_np, layer=-1,
-        nodata_zones=0, nodata_values=1
+        zone_ids=[1, 2, 3], nodata_values=1
     )
     # dask case
     zones_da, _, values_da = create_zones_values(backend='dask')
     df_da = crosstab(
         zones=zones_da, values=values_da, layer=-1,
-        nodata_zones=0, nodata_values=1
+        zone_ids=[1, 2, 3], nodata_values=1
     )
     check_results(df_np, df_da, crosstab_3d_novalues_results)
 
