@@ -11,6 +11,11 @@ import dask.dataframe as dd
 from dask import delayed
 
 from xrspatial.utils import ngjit, validate_arrays
+from xrspatial.utils import ArrayTypeFunctionMapping
+from xrspatial.utils import not_implemented_func
+
+
+TOTAL_COUNT = '_total_count'
 
 
 def _stats_count(data):
@@ -56,77 +61,6 @@ _dask_std = lambda sum_squares, squared_sum, n: np.sqrt((sum_squares - squared_s
 _dask_var = lambda sum_squares, squared_sum, n: (sum_squares - squared_sum/n) / n  # noqa
 
 
-def _zone_cat_data(
-    zones,
-    values,
-    zone_id,
-    nodata_values,
-    cat=None,
-    cat_id=None
-):
-
-    # array backend
-    if isinstance(zones.data, np.ndarray):
-        array_module = np
-    elif isinstance(zones.data, da.Array):
-        array_module = da
-
-    if len(values.shape) == 2:
-        # 2D case
-        conditions = (
-            (zones.data != zone_id)
-            | ~np.isfinite(values.data)  # mask out nan, inf
-            | (values.data == nodata_values)  # mask out nodata_values
-        )
-
-        if cat is not None:
-            conditions |= values.data != cat
-
-        zone_cat_data = array_module.ma.masked_where(conditions, values.data)
-
-    else:
-        # 3D case
-        cat_data = values[cat_id].data
-        cat_masked_data = array_module.ma.masked_invalid(cat_data)
-        zone_cat_data = array_module.ma.masked_where(
-            (
-                (zones.data != zone_id)
-                | (cat_data == nodata_values)
-            ),
-            cat_masked_data
-        )
-    return zone_cat_data
-
-
-def _stats(
-    zones: xr.DataArray,
-    values: xr.DataArray,
-    unique_zones: List[int],
-    stats_funcs: List,
-    nodata_values: Union[int, float],
-) -> Dict:
-
-    stats_dict = {}
-    # zone column
-    stats_dict["zone"] = unique_zones
-    # stats columns
-    for stats in stats_funcs:
-        stats_dict[stats] = []
-
-    for zone_id in unique_zones:
-        # get zone values
-        zone_values = _zone_cat_data(zones, values, zone_id, nodata_values)
-        for stats in stats_funcs:
-            stats_func = stats_funcs.get(stats)
-            if not callable(stats_func):
-                raise ValueError(stats)
-            stats_dict[stats].append(stats_func(zone_values))
-
-    stats_dict["zone"] = unique_zones
-
-    return stats_dict
-
-
 @ngjit
 def _strides(flatten_zones, unique_zones):
     num_elements = flatten_zones.shape[0]
@@ -143,19 +77,19 @@ def _strides(flatten_zones, unique_zones):
     return strides
 
 
-@delayed
-def _stats_func_dask_numpy(
-    zones_block: np.array,
-    values_block: np.array,
-    unique_zones: np.array,
-    zone_ids: np.array,
-    func: Callable,
-    nodata_values: Union[int, float] = None,
-) -> pd.DataFrame:
+def _sort_and_stride(zones, values, unique_zones):
+    flatten_zones = zones.ravel()
+    sorted_indices = np.argsort(flatten_zones)
+    sorted_zones = flatten_zones[sorted_indices]
 
-    sorted_zones = np.sort(zones_block.flatten())
-    sorted_indices = np.argsort(zones_block.flatten())
-    values_by_zones = values_block.flatten()[sorted_indices]
+    values_shape = values.shape
+    if len(values_shape) == 3:
+        values_by_zones = values.reshape(
+            values_shape[0], values_shape[1] * values_shape[2])
+        for i in range(values_shape[0]):
+            values_by_zones[i] = values_by_zones[i][sorted_indices]
+    else:
+        values_by_zones = values.ravel()[sorted_indices]
 
     # exclude nans from calculation
     # flatten_zones is already sorted, NaN elements (if any) are at the end
@@ -163,6 +97,17 @@ def _stats_func_dask_numpy(
     sorted_zones = sorted_zones[np.isfinite(sorted_zones)]
     zone_breaks = _strides(sorted_zones, unique_zones)
 
+    return values_by_zones, zone_breaks
+
+
+def _calc_stats(
+    values_by_zones: np.array,
+    zone_breaks: np.array,
+    unique_zones: np.array,
+    zone_ids: np.array,
+    func: Callable,
+    nodata_values: Union[int, float] = None,
+):
     start = 0
     results = np.full(unique_zones.shape, np.nan)
     for i in range(len(unique_zones)):
@@ -175,7 +120,26 @@ def _stats_func_dask_numpy(
             if len(zone_values) > 0:
                 results[i] = func(zone_values)
         start = end
+    return results
 
+
+@delayed
+def _single_stats_func(
+    zones_block: np.array,
+    values_block: np.array,
+    unique_zones: np.array,
+    zone_ids: np.array,
+    func: Callable,
+    nodata_values: Union[int, float] = None,
+) -> pd.DataFrame:
+
+    values_by_zones, zone_breaks = _sort_and_stride(
+        zones_block, values_block, unique_zones
+    )
+    results = _calc_stats(
+        values_by_zones, zone_breaks,
+        unique_zones, zone_ids, func, nodata_values
+    )
     return results
 
 
@@ -236,7 +200,7 @@ def _stats_dask_numpy(
         stats_func = _DASK_BLOCK_STATS.get(s)
         stats_by_block = [
             da.from_delayed(
-                delayed(_stats_func_dask_numpy)(
+                delayed(_single_stats_func)(
                     z, v, unique_zones, zone_ids, stats_func, nodata_values
                 ), shape=(np.nan,), dtype=dask_dtypes[s]
             )
@@ -288,52 +252,33 @@ def _stats_numpy(
     stats_funcs: Dict,
     nodata_values: Union[int, float],
 ) -> pd.DataFrame:
+
     # find ids for all zones
     unique_zones = np.unique(zones[np.isfinite(zones)])
     # selected zones to do analysis
     if zone_ids is None:
         zone_ids = unique_zones
-    # remove zones that does not exist in `zones` raster
-    zone_ids = [z for z in zone_ids if z in unique_zones]
+    else:
+        zone_ids = np.unique(zone_ids)
+        # remove zones that do not exist in `zones` raster
+        zone_ids = [z for z in zone_ids if z in unique_zones]
+
+    selected_indexes = [i for i, z in enumerate(unique_zones) if z in zone_ids]
+    values_by_zones, zone_breaks = _sort_and_stride(
+        zones, values, unique_zones
+    )
 
     stats_dict = {}
-    # zone column
     stats_dict["zone"] = zone_ids
-    # stats columns
     for stats in stats_funcs:
-        stats_dict[stats] = []
-
-    flatten_zones = zones.flatten()
-    sorted_indices = np.argsort(flatten_zones)
-    sorted_zones = flatten_zones[sorted_indices]
-    values_by_zones = values.flatten()[sorted_indices]
-
-    # exclude nans from calculation
-    # flatten_zones is already sorted, NaN elements (if any) are at the end
-    # of the array, removing them will not affect data before them
-    sorted_zones = sorted_zones[np.isfinite(sorted_zones)]
-    zone_breaks = _strides(sorted_zones, unique_zones)
-
-    start = 0
-    for i in range(len(unique_zones)):
-        end = zone_breaks[i]
-        if unique_zones[i] in zone_ids:
-            zone_values = values_by_zones[start:end]
-            # filter out non-finite and nodata_values
-            zone_values = zone_values[
-                np.isfinite(zone_values) & (zone_values != nodata_values)]
-            if len(zone_values) == 0:
-                stats_dict[stats].append(np.nan)
-            else:
-                for stats in stats_funcs:
-                    stats_func = stats_funcs.get(stats)
-                    if not callable(stats_func):
-                        raise ValueError(stats)
-                    stats_dict[stats].append(stats_func(zone_values))
-        start = end
+        func = stats_funcs.get(stats)
+        stats_dict[stats] = _calc_stats(
+            values_by_zones, zone_breaks,
+            unique_zones, zone_ids, func, nodata_values
+        )
+        stats_dict[stats] = stats_dict[stats][selected_indexes]
 
     stats_df = pd.DataFrame(stats_dict)
-
     return stats_df
 
 
@@ -406,24 +351,37 @@ def stats(
 
     Examples
     --------
-    .. plot::
-       :include-source:
-
-        import numpy as np
-        import xarray as xr
-        from xrspatial.zonal import stats
-
-        height, width = 10, 10
-        # Values raster
-        values = xr.DataArray(np.arange(height * width).reshape(height, width))
-        # Zones raster
-        zones = xr.DataArray(np.zeros(height * width).reshape(height, width))
-        zones[:5, :5] = 0
-        zones[:5, 5:] = 10
-        zones[5:, :5] = 20
-        zones[5:, 5:] = 30
+    stats() works with NumPy backed DataArray
 
     .. sourcecode:: python
+        >>> import numpy as np
+        >>> import xarray as xr
+        >>> from xrspatial.zonal import stats
+        >>> height, width = 10, 10
+        >>> values_data = np.array([
+            [ 0,  1,  2,  3,  4,  5,  6,  7,  8,  9],
+            [10, 11, 12, 13, 14, 15, 16, 17, 18, 19],
+            [20, 21, 22, 23, 24, 25, 26, 27, 28, 29],
+            [30, 31, 32, 33, 34, 35, 36, 37, 38, 39],
+            [40, 41, 42, 43, 44, 45, 46, 47, 48, 49],
+            [50, 51, 52, 53, 54, 55, 56, 57, 58, 59],
+            [60, 61, 62, 63, 64, 65, 66, 67, 68, 69],
+            [70, 71, 72, 73, 74, 75, 76, 77, 78, 79],
+            [80, 81, 82, 83, 84, 85, 86, 87, 88, 89],
+            [90, 91, 92, 93, 94, 95, 96, 97, 98, 99]])
+        >>> values = xr.DataArray(values_data)
+        >>> zones_data = np.array([
+            [ 0.,  0.,  0.,  0.,  0., 10., 10., 10., 10., 10.],
+            [ 0.,  0.,  0.,  0.,  0., 10., 10., 10., 10., 10.],
+            [ 0.,  0.,  0.,  0.,  0., 10., 10., 10., 10., 10.],
+            [ 0.,  0.,  0.,  0.,  0., 10., 10., 10., 10., 10.],
+            [ 0.,  0.,  0.,  0.,  0., 10., 10., 10., 10., 10.],
+            [20., 20., 20., 20., 20., 30., 30., 30., 30., 30.],
+            [20., 20., 20., 20., 20., 30., 30., 30., 30., 30.],
+            [20., 20., 20., 20., 20., 30., 30., 30., 30., 30.],
+            [20., 20., 20., 20., 20., 30., 30., 30., 30., 30.],
+            [20., 20., 20., 20., 20., 30., 30., 30., 30., 30.]])
+        >>> zones = xr.DataArray(zones_data)
 
         >>> # Calculate Stats
         >>> stats_df = stats(zones=zones, values=values)
@@ -446,8 +404,14 @@ def stats(
         2  20     3600
         3  30     3850
 
+    stats() works with Dask with NumPy backed DataArray
+        >>> import dask.array as da
+        >>> import dask.array as da
+        >>> values_dask = xr.DataArray(da.from_array(values, chunks=(3, 3)))
+        >>> zones_dask = xr.DataArray(da.from_array(zones, chunks=(3, 3)))
+
         >>> # Calculate Stats with dask backed xarray DataArrays
-        >>> dask_stats_df = stats(zones=dask_zones, values=dask_values)
+        >>> dask_stats_df = stats(zones=zones_dask, values=values_dask)
         >>> print(type(dask_stats_df))
         <class 'dask.dataframe.core.DataFrame'>
         >>> print(dask_stats_df.compute())
@@ -460,7 +424,7 @@ def stats(
         >>> # Custom Stats with dask backed xarray DataArrays
         >>> dask_custom_stats ={'double_sum': lambda val: val.sum()*2}
         >>> dask_custom_stats_df = stats(
-        >>>      zones=dask_zones, values=dask_values, stats_funcs=custom_stats
+        >>>      zones=zones_dask, values=values_dask, stats_funcs=custom_stats
         >>> )
         >>> print(type(dask_custom_stats_df))
         <class 'dask.dataframe.core.DataFrame'>
@@ -509,152 +473,258 @@ def stats(
     elif isinstance(stats_funcs, dict):
         stats_funcs_dict = stats_funcs.copy()
 
-    if isinstance(values.data, np.ndarray):
-        # numpy case
-        stats_df = _stats_numpy(
-            zones.data,
-            values.data,
-            zone_ids,
-            stats_funcs_dict,
-            nodata_values
-        )
-    else:
-        # dask case
-        stats_df = _stats_dask_numpy(
-            zones.data,
-            values.data,
-            zone_ids,
-            stats_funcs_dict,
-            nodata_values
-        )
-
+    mapper = ArrayTypeFunctionMapping(
+        numpy_func=_stats_numpy,
+        dask_func=_stats_dask_numpy,
+        cupy_func=lambda *args: not_implemented_func(
+            *args, messages='stats() does not support cupy backed DataArray'
+        ),
+        dask_cupy_func=lambda *args: not_implemented_func(
+            *args, messages='stats() does not support dask with cupy backed DataArray'  # noqa
+        ),
+    )
+    stats_df = mapper(values)(
+        zones.data, values.data, zone_ids, stats_funcs_dict, nodata_values
+    )
     return stats_df
 
 
-def _crosstab_dict(zones, values, unique_zones, cats, nodata_values, agg):
+def _find_cats(values, cat_ids, nodata_values):
+    if len(values.shape) == 2:
+        # 2D case
+        unique_cats = np.unique(values.data[
+            np.isfinite(values.data) & (values.data != nodata_values)
+        ])
+    else:
+        # 3D case
+        unique_cats = values[values.dims[0]].data
 
-    crosstab_dict = {}
-    crosstab_dict["zone"] = unique_zones
+    if cat_ids is None:
+        cat_ids = unique_cats
+    else:
+        if isinstance(values.data, np.ndarray):
+            # remove cats that do not exist in `values` raster
+            cat_ids = [c for c in cat_ids if c in unique_cats]
+        else:
+            cat_ids = _select_ids(unique_cats, cat_ids)
 
-    for i in cats:
-        crosstab_dict[i] = []
+    return unique_cats, cat_ids
 
-    for cat_id, cat in enumerate(cats):
-        for zone_id in unique_zones:
-            # get category cat values in the selected zone
-            zone_cat_data = _zone_cat_data(
-                zones, values, zone_id, nodata_values, cat, cat_id
-            )
-            zone_cat_count = _stats_count(zone_cat_data)
-            crosstab_dict[cat].append(zone_cat_count)
 
-    if agg == "percentage":
-        zone_counts = _stats(
-            zones,
-            values,
-            unique_zones,
-            {"count": _stats_count},
-            nodata_values
-        )["count"]
-        for c, cat in enumerate(cats):
-            for z in range(len(unique_zones)):
-                crosstab_dict[cat][z] = (
-                    crosstab_dict[cat][z] / zone_counts[z] * 100
-                )  # noqa
+def _get_zone_values(values_by_zones, start, end):
+    if len(values_by_zones.shape) == 1:
+        # 1D flatten, i.e, original data is 2D
+        return values_by_zones[start:end]
+    else:
+        # 2D flatten, i.e, original data is 3D
+        return values_by_zones[:, start:end]
 
-    return crosstab_dict
+
+def _single_zone_crosstab(
+    zone_values,
+    unique_cats,
+    cat_ids,
+    nodata_values,
+    crosstab_dict,
+):
+    if len(zone_values.shape) == 1:
+        # 1D flatten, i.e, original data is 2D
+
+        # filter out non-finite and nodata_values
+        zone_values = zone_values[
+            np.isfinite(zone_values) & (zone_values != nodata_values)
+        ]
+        total_count = zone_values.shape[0]
+        crosstab_dict[TOTAL_COUNT].append(total_count)
+
+        sorted_zone_values = np.sort(zone_values)
+        zone_cat_breaks = _strides(sorted_zone_values, unique_cats)
+
+        cat_start = 0
+
+        for j, cat in enumerate(unique_cats):
+            if cat in cat_ids:
+                count = zone_cat_breaks[j] - cat_start
+                crosstab_dict[cat].append(count)
+                cat_start = zone_cat_breaks[j]
+
+    else:
+        # 2D flatten, i.e, original data is 3D
+        for j, cat in enumerate(unique_cats):
+            if cat in cat_ids:
+                zone_cat_data = zone_values[j]
+                # filter out non-finite and nodata_values
+                zone_cat_data = zone_cat_data[
+                    np.isfinite(zone_cat_data)
+                    & (zone_cat_data != nodata_values)
+                ]
+                count = zone_cat_data.shape[0]
+                crosstab_dict[cat].append(count)
 
 
 def _crosstab_numpy(
-    zones,
-    values,
-    zone_ids,
-    cat_ids,
-    nodata_zones,
-    nodata_values,
-    agg
-):
-
-    if cat_ids is not None:
-        cats = np.array(cat_ids)
-    else:
-        # no categories provided, find all possible cats in values raster
-        if len(values.shape) == 3:
-            # 3D case
-            cats = values.indexes[values.dims[0]].values
-        else:
-            # 2D case
-            # mask out all invalid values such as: nan, inf
-            cats = da.unique(values.data[da.isfinite(values.data)]).compute()
-            cats = sorted(list(set(cats) - set([nodata_values])))
+    zones: np.ndarray,
+    values: np.ndarray,
+    zone_ids: List[Union[int, float]],
+    unique_cats: np.ndarray,
+    cat_ids: Union[List, np.ndarray],
+    nodata_values: Union[int, float],
+    agg: str,
+) -> pd.DataFrame:
 
+    # find ids for all zones
+    unique_zones = np.unique(zones[np.isfinite(zones)])
+    # selected zones to do analysis
     if zone_ids is None:
-        # do not consider zone with nodata values
-        unique_zones = np.unique(zones.data[np.isfinite(zones.data)])
-        unique_zones = sorted(list(set(unique_zones) - set([nodata_zones])))
+        zone_ids = unique_zones
     else:
-        unique_zones = np.array(zone_ids)
+        # remove zones that do not exist in `zones` raster
+        zone_ids = [z for z in zone_ids if z in unique_zones]
 
-    crosstab_dict = _crosstab_dict(
-        zones, values, unique_zones, cats, nodata_values, agg
+    crosstab_dict = {}
+    crosstab_dict["zone"] = zone_ids
+    if len(values.shape) == 2:
+        crosstab_dict[TOTAL_COUNT] = []
+    for cat in cat_ids:
+        crosstab_dict[cat] = []
+
+    values_by_zones, zone_breaks = _sort_and_stride(
+        zones, values, unique_zones
     )
 
+    start = 0
+    for i in range(len(unique_zones)):
+        end = zone_breaks[i]
+        if unique_zones[i] in zone_ids:
+            # get data for zone unique_zones[i]
+            zone_values = _get_zone_values(values_by_zones, start, end)
+            _single_zone_crosstab(
+                zone_values, unique_cats, cat_ids,
+                nodata_values, crosstab_dict
+            )
+        start = end
+
+    if TOTAL_COUNT in crosstab_dict:
+        crosstab_dict[TOTAL_COUNT] = np.array(
+            crosstab_dict[TOTAL_COUNT], dtype=np.float32
+        )
+    for cat in cat_ids:
+        crosstab_dict[cat] = np.array(crosstab_dict[cat])
+
+    # construct output dataframe
+    if agg == 'percentage':
+        # replace 0s with nans to avoid dividing by 0 error
+        crosstab_dict[TOTAL_COUNT][crosstab_dict[TOTAL_COUNT] == 0] = np.nan
+        for cat in cat_ids:
+            crosstab_dict[cat] = crosstab_dict[cat] / crosstab_dict[TOTAL_COUNT] * 100  # noqa
+
     crosstab_df = pd.DataFrame(crosstab_dict)
-
-    # name columns
-    crosstab_df.columns = crosstab_dict.keys()
-
+    crosstab_df = crosstab_df[['zone'] + list(cat_ids)]
     return crosstab_df
 
 
-def _crosstab_dask(
-    zones,
-    values,
-    zone_ids,
+@delayed
+def _single_chunk_crosstab(
+    zones_block: np.array,
+    values_block: np.array,
+    unique_zones: np.array,
+    zone_ids: np.array,
+    unique_cats,
     cat_ids,
-    nodata_zones,
-    nodata_values,
-    agg
+    nodata_values: Union[int, float],
 ):
+    results = {}
+    if len(values_block.shape) == 2:
+        results[TOTAL_COUNT] = []
+    for cat in cat_ids:
+        results[cat] = []
 
-    if cat_ids is not None:
-        cats = np.array(cat_ids)
-    else:
-        # no categories provided, find all possible cats in values raster
-        if len(values.shape) == 3:
-            # 3D case
-            cats = values.indexes[values.dims[0]].values
-        else:
-            # 2D case
-            # precompute categories
-            cats = da.unique(values.data[da.isfinite(values.data)]).compute()
-            cats = sorted(list(set(cats) - set([nodata_values])))
+    values_by_zones, zone_breaks = _sort_and_stride(
+        zones_block, values_block, unique_zones
+    )
 
+    start = 0
+    for i in range(len(unique_zones)):
+        end = zone_breaks[i]
+        if unique_zones[i] in zone_ids:
+            # get data for zone unique_zones[i]
+            zone_values = _get_zone_values(values_by_zones, start, end)
+            _single_zone_crosstab(
+                zone_values, unique_cats, cat_ids,
+                nodata_values, results
+            )
+        start = end
+
+    if TOTAL_COUNT in results:
+        results[TOTAL_COUNT] = np.array(results[TOTAL_COUNT], dtype=np.float32)
+    for cat in cat_ids:
+        results[cat] = np.array(results[cat])
+
+    return results
+
+
+@delayed
+def _select_ids(unique_ids, ids):
+    selected_ids = []
+    for i in ids:
+        if i in unique_ids:
+            selected_ids.append(i)
+    return selected_ids
+
+
+@delayed
+def _crosstab_df_dask(crosstab_by_block, zone_ids, cat_ids, agg):
+    result = crosstab_by_block[0]
+    for i in range(1, len(crosstab_by_block)):
+        for k in crosstab_by_block[i]:
+            result[k] += crosstab_by_block[i][k]
+
+    if agg == 'percentage':
+        # replace 0s with nans to avoid dividing by 0 error
+        result[TOTAL_COUNT][result[TOTAL_COUNT] == 0] = np.nan
+        for cat in cat_ids:
+            result[cat] = result[cat] / result[TOTAL_COUNT] * 100
+
+    df = pd.DataFrame(result)
+    df['zone'] = zone_ids
+    columns = ['zone'] + list(cat_ids)
+    df = df[columns]
+    return df
+
+
+def _crosstab_dask_numpy(
+    zones: np.ndarray,
+    values: np.ndarray,
+    zone_ids: List[Union[int, float]],
+    unique_cats: np.ndarray,
+    cat_ids: Union[List, np.ndarray],
+    nodata_values: Union[int, float],
+    agg: str,
+):
+    # find ids for all zones
+    unique_zones = np.unique(zones[np.isfinite(zones)])
     if zone_ids is None:
-        # precompute unique zones
-        unique_zones = da.unique(zones.data[da.isfinite(zones.data)]).compute()
-        # do not consider zone with nodata values
-        unique_zones = sorted(list(set(unique_zones) - set([nodata_zones])))
+        zone_ids = unique_zones
     else:
-        unique_zones = np.array(zone_ids)
+        zone_ids = _select_ids(unique_zones, zone_ids)
 
-    crosstab_dict = _crosstab_dict(
-        zones, values, unique_zones, cats, nodata_values, agg
+    cat_ids = _select_ids(unique_cats, cat_ids)
+
+    zones_blocks = zones.to_delayed().ravel()
+    values_blocks = values.to_delayed().ravel()
+
+    crosstab_by_block = [
+        _single_chunk_crosstab(
+            z, v, unique_zones, zone_ids,
+            unique_cats, cat_ids, nodata_values
+        )
+        for z, v in zip(zones_blocks, values_blocks)
+    ]
+
+    crosstab_df = _crosstab_df_dask(
+        crosstab_by_block, zone_ids, cat_ids, agg
     )
-    crosstab_dict = {
-        stats: da.stack(zonal_stats, axis=0)
-        for stats, zonal_stats in crosstab_dict.items()
-    }
-
-    # generate dask dataframe
-    crosstab_df = dd.concat(
-        [dd.from_dask_array(stats) for stats in crosstab_dict.values()], axis=1
-    )
-
-    # name columns
-    crosstab_df.columns = crosstab_dict.keys()
-
-    return crosstab_df
+    return dd.from_delayed(crosstab_df)
 
 
 def crosstab(
@@ -664,7 +734,6 @@ def crosstab(
     cat_ids: List[Union[int, float]] = None,
     layer: Optional[int] = None,
     agg: Optional[str] = "count",
-    nodata_zones: Optional[Union[int, float]] = None,
     nodata_values: Optional[Union[int, float]] = None,
 ) -> Union[pd.DataFrame, dd.DataFrame]:
     """
@@ -716,11 +785,6 @@ def crosstab(
         If the data is 2D, available options are: percentage, count.
         If the data is 3D, available option is: count.
 
-    nodata_zones: int, float, default=None
-        Nodata value in `zones` raster.
-        Cells with `nodata` do not belong to any zone,
-        and thus excluded from calculation.
-
     nodata_values: int, float, default=None
         Nodata value in `values` raster.
         Cells with `nodata` do not belong to any zone,
@@ -736,33 +800,27 @@ def crosstab(
 
     Examples
     --------
-    .. plot::
-       :include-source:
-
-        import dask.array as da
-        import numpy as np
-        import xarray as xr
-        from xrspatial.zonal import crosstab
-
-        values_data = np.asarray([[0, 0, 10, 20],
-                                  [0, 0, 0, 10],
-                                  [0, np.nan, 20, 50],
-                                  [10, 30, 40, np.inf],
-                                  [10, 10, 50, 0]])
-        values = xr.DataArray(values_data)
-
-        zones_data = np.asarray([[1, 1, 6, 6],
-                                 [1, np.nan, 6, 6],
-                                 [3, 5, 6, 6],
-                                 [3, 5, 7, np.nan],
-                                 [3, 7, 7, 0]])
-        zones = xr.DataArray(zones_data)
-
-        values_dask = xr.DataArray(da.from_array(values, chunks=(3, 3)))
-        zones_dask = xr.DataArray(da.from_array(zones, chunks=(3, 3)))
+    crosstab() works with NumPy backed DataArray.
 
     .. sourcecode:: python
+        >>> import numpy as np
+        >>> import xarray as xr
+        >>> from xrspatial.zonal import crosstab
 
+        >>> values_data = np.asarray([
+            [0, 0, 10, 20],
+            [0, 0, 0, 10],
+            [0, np.nan, 20, 50],
+            [10, 30, 40, np.inf],
+            [10, 10, 50, 0]])
+        >>> values = xr.DataArray(values_data)
+        >>> zones_data = np.asarray([
+            [1, 1, 6, 6],
+            [1, np.nan, 6, 6],
+            [3, 5, 6, 6],
+            [3, 5, 7, np.nan],
+            [3, 7, 7, 0]])
+        >>> zones = xr.DataArray(zones_data)
         >>> # Calculate Crosstab, numpy case
         >>> df = crosstab(zones=zones, values=values)
         >>> print(df)
@@ -774,7 +832,12 @@ def crosstab(
             4      6    1     2     2     0     0     1
             5      7    0     1     0     0     1     1
 
-        >>> # Calculate Crosstab, dask case
+    crosstab() works with Dask with NumPy backed DataArray.
+
+    .. sourcecode:: python
+        >>> import dask.array as da
+        >>> values_dask = xr.DataArray(da.from_array(values, chunks=(3, 3)))
+        >>> zones_dask = xr.DataArray(da.from_array(zones, chunks=(3, 3)))
         >>> df = crosstab(zones=zones_dask, values=values_dask)
         >>> print(df)
             Dask DataFrame Structure:
@@ -795,6 +858,7 @@ def crosstab(
             4      6    1     2     2     0     0     1
             5      7    0     1     0     0     1     1
     """
+
     if not isinstance(zones, xr.DataArray):
         raise TypeError("zones must be instance of DataArray")
 
@@ -805,13 +869,13 @@ def crosstab(
         raise ValueError("zones must be 2D")
 
     if not (
-        issubclass(zones.data.dtype.type, np.integer)
-        or issubclass(zones.data.dtype.type, np.floating)
+            issubclass(zones.data.dtype.type, np.integer)
+            or issubclass(zones.data.dtype.type, np.floating)
     ):
         raise ValueError("`zones` must be an xarray of integers or floats")
 
     if not issubclass(values.data.dtype.type, np.integer) and not issubclass(
-        values.data.dtype.type, np.floating
+            values.data.dtype.type, np.floating
     ):
         raise ValueError("`values` must be an xarray of integers or floats")
 
@@ -848,17 +912,37 @@ def crosstab(
         if zones.shape != values.shape[1:]:
             raise ValueError("Incompatible shapes")
 
-    if isinstance(values.data, np.ndarray):
-        # numpy case
-        crosstab_df = _crosstab_numpy(
-            zones, values, zone_ids, cat_ids, nodata_zones, nodata_values, agg
-        )
-    else:
-        # dask case
-        crosstab_df = _crosstab_dask(
-            zones, values, zone_ids, cat_ids, nodata_zones, nodata_values, agg
-        )
+        if isinstance(values.data, da.Array):
+            # dask case, rechunk if necessary
+            zones_chunks = zones.chunks
+            expected_values_chunks = {
+                0: (values.shape[0],),
+                1: zones_chunks[0],
+                2: zones_chunks[1],
+            }
+            actual_values_chunks = {
+                i: values.chunks[i] for i in range(3)
+            }
+            if actual_values_chunks != expected_values_chunks:
+                values.data = values.data.rechunk(expected_values_chunks)
 
+    # find categories
+    unique_cats, cat_ids = _find_cats(values, cat_ids, nodata_values)
+
+    mapper = ArrayTypeFunctionMapping(
+        numpy_func=_crosstab_numpy,
+        dask_func=_crosstab_dask_numpy,
+        cupy_func=lambda *args: not_implemented_func(
+            *args, messages='crosstab() does not support cupy backed DataArray'
+        ),
+        dask_cupy_func=lambda *args: not_implemented_func(
+            *args, messages='crosstab() does not support dask with cupy backed DataArray'  # noqa
+        ),
+    )
+    crosstab_df = mapper(values)(
+        zones.data, values.data,
+        zone_ids, unique_cats, cat_ids, nodata_values, agg
+    )
     return crosstab_df
 
 
@@ -895,13 +979,16 @@ def apply(
     Examples
     --------
     .. sourcecode:: python
-
-        >>> zones_val = np.array([[1, 1, 0, 2],
-        >>>                       [0, 2, 1, 2]])
-        >>> zones = xarray.DataArray(zones_val)
-        >>> values_val = np.array([[2, -1, 5, 3],
-        >>>                        [3, np.nan, 20, 10]])
-        >>> agg = xarray.DataArray(values_val)
+        >>> import numpy as np
+        >>> import xarray as xr
+        >>> zones_val = np.array([
+            [1, 1, 0, 2],
+            [0, 2, 1, 2]])
+        >>> zones = xr.DataArray(zones_val)
+        >>> values_val = np.array([
+            [2, -1, 5, 3],
+            [3, np.nan, 20, 10]])
+        >>> agg = xr.DataArray(values_val)
         >>> func = lambda x: 0
         >>> apply(zones, agg, func)
         >>> agg
